import machine
import utime
from sh1106 import SH1106_I2C

mode = 'start' # Mode of illumination
LED_row = 'LED_1' #Selection of LED-row
Data = [] #List with the day/night-curve data 
mode_set = 'start_set' #saves the mode of the previous while-loop

s = 0 #parameter for change_order function
row = 1 #row from the .csv-file
row_total = 0 #total count of rows from .csv-file
csv_step = 10
timer = 0 #difference between previous and current datetime
conversion_factor = 0.4 * 4.6 #Conversion factor for the photon flux density
photon_flux_density1 = 0 #Photon flux density for LED1
photon_flux_density2 = 0 #Photon flux density for LED2
PWM_16bit1 = 0 #PWM of LED1 converted in 16bit-Code
PWM_16bit2 = 0 #PWM of LED2 converted in 16bit-Code
PWM_pro_old_1 = 0 #previous PWM Wert von LED1
PWM_pro_old_2 = 0 #previous PWM Wert von LED2 
PWM_pro1 = 0 #PWM value of LED1
PWM_pro2 = 0 #PWM value of LED2
PWM_from_graph1 = 0 #Converted PWM from regressionline for LED1
PWM_from_graph2 = 0 #Converted PWM from regressionline for LED2

#time variable
time_next = 0 #Day/night
time_now = 0 #Day/night
time_since_start = 0 #set time
time_hr = 0 #localtime as tuple
Datetime_now = 0 #Datetime of current state
Datetime_next = 0 #Datetime of previous state

#Regression for 6LEDs with 300mA constant current source
slope = 16.605 #[PPPFD/%PWM]
y_intersection = -0.9846 #[PPFD]

#Define the pins for control of the PWM
led_1 = machine.PWM(machine.Pin(16)) #PIN: GPIO16)
led_1.freq(1000)
led_2 = machine.PWM(machine.Pin(17)) #PIN: GPIO17)
led_2.freq(1000)

#Define the I2C control of the OLED
sdaPIN=machine.Pin(18)
sclPIN=machine.Pin(19)
i2c=machine.I2C(1,sda=sdaPIN, scl=sclPIN, freq=400000)

#Define button control + Counter
button_mode = machine.Pin(2, machine.Pin.IN, machine.Pin.PULL_DOWN)
button_10 = machine.Pin(3, machine.Pin.IN, machine.Pin.PULL_DOWN)
button_1 = machine.Pin(5, machine.Pin.IN, machine.Pin.PULL_DOWN)
button_LED = machine.Pin(4, machine.Pin.IN, machine.Pin.PULL_DOWN)

count_10_LED1 = 0
count_1_LED1 = 0
count_10_LED2 = 0
count_1_LED2 = 0

#Connecting the OLED display
oled = SH1106_I2C(128, 64, i2c)
oled.fill(0)

#Change the order of values of a given list
def change_order(string, split_char, *args): 
    string = string.split(split_char)

    new_list = []

    for s in args:
        try:
            new_list.append(string[s])  # change to s-1 for abstracted parameters
        except IndexError:
            return False

    return split_char.join(new_list)

#time difference between two dates, returns value in secs
def time_between(time_next, time_now):
    time_next += (0, 0)  # ensure a time past midnight
    time_now += (0, 0)
    return utime.mktime(time_next) - utime.mktime(time_now)

#check the input_values of the mode button
def check_button_mode(mode,time_since_start):
    if button_mode.value() == 1:
        if mode == 'constant':
            mode = 'daynight'
        elif mode == 'start':
            mode = 'constant'
            time_since_start = utime.ticks_ms()+500
        elif mode == 'daynight':
            mode = 'start'
            time_since_start = utime.ticks_ms()+500
        print(mode)    
        utime.sleep(0.5)
    return mode, time_since_start

#Check the input_value for the button performing the counting
def check_button(count_x,button_x):   
    if button_x.value() == 1:
        count_x = button_counter(count_x)

    return count_x

#Conversion of given value from the .csv into a PWM value in 16bit notation
def Convert_PFD_into_PWM(PWM_pro,photon_flux_density,Data,conversion_factor,slope,y_intersection,PWM_from_graph,PWM_16bit,row):
    photon_flux_density = float(Data[row][1]) * conversion_factor * PWM_pro/100
    PWM_from_graph = (photon_flux_density - y_intersection) / slope
    
    if PWM_from_graph < 0:
        PWM_16bit = 0
    
    if PWM_from_graph >= 100:
        PWM_16bit = 65025
            
    if 0 <= PWM_from_graph < 100:
        PWM_16bit = int(65025*PWM_from_graph/100)       
    return PWM_16bit, photon_flux_density

#Check the LED button choice 
def check_button_LED(LED_row):   
    if button_LED.value() == 1:
        if LED_row == 'LED_1':
            LED_row = 'LED_2'
        elif LED_row == 'LED_2':
            LED_row = 'LED_1'
        utime.sleep(0.5)
    return LED_row

#increasing counter
def button_counter(count):
        count = count + 1
        utime.sleep(0.5)
        return count

#combine both counter values into one percentage value (0-100%)
def Counter_into_Percent(count_10,count_1,Total_count):
    
    Total_count = count_10 * 10 + count_1 * 1
    if Total_count > 100:
        Total_count = 0
        count_1 = 0
        count_10 = 0
    return Total_count, count_1, count_10 

#Check the PWM buttons and combine them 
def LED_counter(count_1,count_10,PWM_pro):
        count_1 = check_button(count_1,button_1)
        count_10 = check_button(count_10,button_10)
            
        PWM_pro, count_1, count_10 = Counter_into_Percent(count_10,count_1,PWM_pro)
        return PWM_pro, count_1, count_10

#"Startdisplay"
def OLED_setting(mode):
    
    oled.fill(0)
    oled.text(str(mode),0, 0)      


#Boot-display, while reading csv file
oled.fill(0)
oled.text("the system",15, 25)
oled.text("is booting...",15, 35)
oled.show()

#read csv file
with open('Almeria.csv','r') as file:
    for line in file:
        #line=file.readline()
        line=line.replace("\'", "")
        line=line.replace("Dec", "12") #supplement code here for different weather data
        line=line.replace(",", ".")
        line=line.replace("-", ",")
        line=line.replace(":", ",")
        line=line.replace(" ", ",")
        line=line.rstrip('\n')
        #line=line.rstrip('\r')
        if row_total % csv_step == 0:
            Data.append(line.split(';'))
            row_total += 1
        else:
            row_total += 1


#Script
while True:
    #Check mode and LED row buttons
    mode, time_since_start = check_button_mode(mode,time_since_start)
    LED_row = check_button_LED(LED_row)

    #Mode 'Start'
    if mode =='start':
        OLED_setting(mode)
        #PWM for LED 1, if LED_1
        if LED_row == 'LED_1':
            PWM_pro1, count_1_LED1, count_10_LED1 = LED_counter(count_1_LED1,count_10_LED1,PWM_pro1)
            oled.text('_____',0, 33)
        #PWM for LED 2, if LED_2
        elif LED_row == 'LED_2':
            PWM_pro2, count_1_LED2, count_10_LED2 = LED_counter(count_1_LED2,count_10_LED2,PWM_pro2)
            oled.text('_____',65, 33)
        mode_set = 'start_set'    
        led_1.duty_u16(0)
        led_2.duty_u16(0)
        oled.text('waiting...',0, 15)
        oled.text('LED_1',0, 30)
        oled.text('LED_2',65, 30)
        oled.text(str(PWM_pro1),0, 45)
        oled.text(str(PWM_pro2),65, 45)
        oled.show()
    #Mode 'constant'
    if mode == 'constant':
        OLED_setting(mode)
        #PWM for LED 1, if LED_1
        if LED_row == 'LED_1':
            PWM_pro1, count_1_LED1, count_10_LED1 = LED_counter(count_1_LED1,count_10_LED1,PWM_pro1)
            oled.text('_____',0, 33)
        #PWM for LED 2, if LED_2
        elif LED_row == 'LED_2':
            PWM_pro2, count_1_LED2, count_10_LED2 = LED_counter(count_1_LED2,count_10_LED2,PWM_pro2)
            oled.text('_____',65, 33)
        #Adjusting the display
        #Timer, plus change tuple-element day from 1 to zero
        time_hr = utime.localtime(int((utime.ticks_ms()-time_since_start)/(1000)))
        time_hr_list = list(time_hr)
        time_hr_list[2] = time_hr_list[2] - 1
        time_hr = tuple(time_hr_list)
        oled.text(str(time_hr[2:6]).replace(",",":"),0, 15)
        #Display both LEDS + the PWM value
        oled.text('LED_1',0, 30)
        oled.text('LED_2',65, 30)
        oled.text(str(PWM_pro1),0, 45)
        oled.text(str(PWM_pro2),65, 45)
        oled.show()
        
        #Swapping to a new PWM solely when the value changes or the mode has changed
        if PWM_pro_old_1 != PWM_pro1 or PWM_pro_old_2 != PWM_pro2 or mode_set == 'start_set':
            
            led_1.duty_u16(int(PWM_pro1*65025/100))
            led_2.duty_u16(int(PWM_pro2*65025/100))
            PWM_pro_old_1 = PWM_pro1
            PWM_pro_old_2 = PWM_pro2
            mode_set = 'constant_set'
    
    #Mode 'day/night'
    if mode == 'daynight':
        OLED_setting(mode)
        if mode_set == 'constant_set':
            led_1.duty_u16(0)
            led_2.duty_u16(0) 
        #PWM for LED 1, if LED_1
        if LED_row == 'LED_1':
            PWM_pro1, count_1_LED1, count_10_LED1 = LED_counter(count_1_LED1,count_10_LED1,PWM_pro1)
            oled.text('_____',0, 33)
        #PWM for LED 2, if LED_2
        elif LED_row == 'LED_2':
            PWM_pro2, count_1_LED2, count_10_LED2 = LED_counter(count_1_LED2,count_10_LED2,PWM_pro2)
            oled.text('_____',65, 33)
        #Adjusting the display 
        oled.text('LED_1',0, 30)
        oled.text('LED_2',65, 30)
        oled.text(str(PWM_pro1),0, 45)
        oled.text(str(PWM_pro2),65, 45)
        
        #timer adjustment from differences between the date times
        if row == row_total/csv_step:
            row = 1
        
        if row < ((row_total/csv_step) - 1):
            Datetime_now = eval(change_order(Data[row][0], ',', 2,1,0,3,4,5))
            Datetime_next = eval(change_order(Data[row+1][0], ',', 2,1,0,3,4,5))
            timer = time_between(Datetime_next, Datetime_now)
        
        if row == (row_total/csv_step) - 1:
            Datetime_now = eval(change_order(Data[1][0], ',', 2,1,0,3,4,5))
            Datetime_next = eval(change_order(Data[row][0], ',', 2,1,0,3,4,5))
            timer = 24 * 3600 - time_between(Datetime_next, Datetime_now)
        
        #display the current Daytime
        oled.text(Data[row][0].replace(",",":")[11:16],0, 15)
        
        #adjusting the new values from the .csv when the time is greater than the timer
        if (utime.ticks_ms() - time_since_start)/1000 >  timer:
            
            PWM_16bit1, photon_flux_density1 = Convert_PFD_into_PWM(PWM_pro1,photon_flux_density1,Data,conversion_factor,slope,y_intersection,PWM_from_graph1,PWM_16bit1,row)
            PWM_16bit2, photon_flux_density2 = Convert_PFD_into_PWM(PWM_pro2,photon_flux_density2,Data,conversion_factor,slope,y_intersection,PWM_from_graph2,PWM_16bit2,row)
            led_1.duty_u16(PWM_16bit1)
            led_2.duty_u16(PWM_16bit2)
            time_since_start = utime.ticks_ms()
            row = row + 1
            mode_set = 'day/night_set'
        
        #display PFD of both LEDs 
        oled.text(str(int(photon_flux_density1)),30,45)
        oled.text(str(int(photon_flux_density2)),95,45)
        oled.show()